// src/shared/polyfills.ts
async function installURLPattern() {
  if (!globalThis.URLPattern) {
    const { URLPattern } = await import("urlpattern-polyfill/urlpattern");
    globalThis.URLPattern = URLPattern;
  }
}

// src/node/app/http/errors.ts
function invariant(value, message) {
  if (value === false || value === null || typeof value === "undefined") {
    throw new httpError(400, message);
  }
}
function validate(condition, message) {
  if (!condition)
    throw new httpError(422, message);
}
function httpError(statusCode, message) {
  throw new HTTPError(statusCode, message);
}
function httpJSONError(statusCode, data = {}) {
  throw new HTTPJSONError(statusCode, data);
}
var HTTPError = class extends Error {
  constructor(statusCode, message) {
    super(message);
    this.statusCode = statusCode;
  }
};
var HTTPJSONError = class extends Error {
  constructor(statusCode, data) {
    super();
    this.statusCode = statusCode;
    this.data = data;
  }
};
function isHTTPError(error) {
  return error instanceof HTTPError;
}
function isHTTPJSONError(error) {
  return error instanceof HTTPJSONError;
}

// src/node/app/http/http-bridge.ts
import * as set_cookie_parser from "set-cookie-parser";
async function getRequest(base, req) {
  let headers = req.headers;
  if (req.httpVersionMajor === 2) {
    headers = Object.assign({}, headers);
    delete headers[":method"];
    delete headers[":path"];
    delete headers[":authority"];
    delete headers[":scheme"];
  }
  return new Request(base + req.url, {
    method: req.method,
    headers,
    body: getRawBody(req)
  });
}
function getRawBody(req) {
  const h = req.headers;
  if (!h["content-type"]) {
    return null;
  }
  const length = Number(h["content-length"]);
  if (isNaN(length) && h["transfer-encoding"] == null) {
    return null;
  }
  if (req.destroyed) {
    const readable = new ReadableStream();
    readable.cancel();
    return readable;
  }
  let size = 0;
  let cancelled = false;
  return new ReadableStream({
    start(controller) {
      req.on("error", (error) => {
        controller.error(error);
      });
      req.on("end", () => {
        if (cancelled)
          return;
        controller.close();
      });
      req.on("data", (chunk) => {
        if (cancelled)
          return;
        size += chunk.length;
        if (size > length) {
          controller.error(new Error("content-length exceeded"));
          return;
        }
        controller.enqueue(chunk);
        if (controller.desiredSize === null || controller.desiredSize <= 0) {
          req.pause();
        }
      });
    },
    pull() {
      req.resume();
    },
    cancel(reason) {
      cancelled = true;
      req.destroy(reason);
    }
  });
}
async function setResponse(res, response) {
  const headers = Object.fromEntries(response.headers);
  if (response.headers.has("set-cookie")) {
    const header = response.headers.get("set-cookie");
    const split = set_cookie_parser.splitCookiesString(header);
    headers["set-cookie"] = split;
  }
  res.writeHead(response.status, headers);
  if (!response.body) {
    res.end();
    return;
  }
  const reader = response.body.getReader();
  if (res.destroyed) {
    reader.cancel();
    return;
  }
  const cancel = (error) => {
    res.off("close", cancel);
    res.off("error", cancel);
    reader.cancel(error).catch(() => {
    });
    if (error)
      res.destroy(error);
  };
  res.on("close", cancel);
  res.on("error", cancel);
  next();
  async function next() {
    try {
      for (; ; ) {
        const { done, value } = await reader.read();
        if (done)
          break;
        if (!res.write(value)) {
          res.once("drain", next);
          return;
        }
      }
      res.end();
    } catch (error) {
      cancel(error instanceof Error ? error : new Error(String(error)));
    }
  }
}

// src/node/app/http/request.ts
async function handleRequest(request, urlPattern, getClientAddress, loader, methods) {
  var _a;
  const methodOverride = request.method === "POST" ? (await request.formData()).get("_method") : null;
  const method = typeof methodOverride === "string" ? methodOverride : request.method;
  if (methods && !methods.includes(method))
    throw httpError(404, "Not found");
  const url = new URL(request.url);
  if (!urlPattern.test(url))
    throw httpError(404, "Not found");
  const mod = await loader();
  let handler = mod[method];
  if (!handler && method === "HEAD")
    handler = mod.GET;
  if (!handler)
    handler = mod.ANY;
  if (!handler) {
    return new Response(`${method} method not allowed`, {
      status: 405,
      headers: {
        allow: getAllowedMethods(mod).join(", ")
      }
    });
  }
  const headers = {};
  const cookies = [];
  const params = ((_a = urlPattern.exec(url)) == null ? void 0 : _a.pathname.groups) ?? {};
  const response = await handler(
    createRequestEvent(
      request,
      url,
      params,
      getClientAddress,
      headers,
      cookies
    )
  );
  if (!(response instanceof Response)) {
    throw new Error(
      `Invalid return value from route handler at ${url.pathname}. Should return a \`Response\`.`
    );
  }
  for (const name in headers) {
    const value = headers[name];
    if (value !== null)
      response.headers.set(name, value + "");
  }
  for (const cookie of cookies) {
    response.headers.append("set-cookie", cookie);
  }
  return response;
}
function createRequestEvent(request, url, params, getClientAddress, headers = {}, cookies = []) {
  return {
    get url() {
      return url;
    },
    get clientAddress() {
      const address = getClientAddress();
      if (typeof address !== "string") {
        throw new Error("Could not determine `clientAddress`");
      }
      return address;
    },
    get params() {
      return params;
    },
    get request() {
      return request;
    },
    setHeaders(newHeaders) {
      for (const key in newHeaders) {
        const name = key.toLowerCase();
        const value = newHeaders[key];
        if (name === "set-cookie") {
          const newCookies = Array.isArray(value) ? value : value ? [value + ""] : [];
          for (const cookie of newCookies) {
            if (cookies.includes(cookie)) {
              throw new Error(
                `"${key}" header already has cookie with same value`
              );
            }
            cookies.push(cookie);
          }
        } else if (name in headers) {
          throw new Error(`"${key}" header is already set`);
        } else if (typeof value !== "string") {
          throw new Error(
            `"${key}" value must be a string (found ${typeof value}).`
          );
        } else {
          headers[name] = value;
        }
      }
    }
  };
}
function getAllowedMethods(mod) {
  const allowed = [];
  for (const method of HTTP_METHODS) {
    if (method in mod)
      allowed.push(method);
  }
  if (mod.GET || mod.HEAD)
    allowed.push("HEAD");
  return allowed;
}

// src/node/app/http/response.ts
var json = (data, init = {}) => {
  const responseInit = typeof init === "number" ? { status: init } : init;
  const headers = new Headers(responseInit.headers);
  if (!headers.has("Content-Type")) {
    headers.set("Content-Type", "application/json; charset=utf-8");
  }
  return new Response(JSON.stringify(data), {
    ...responseInit,
    headers
  });
};
var redirect = (url, init = 302) => {
  let responseInit = init;
  if (typeof responseInit === "number") {
    responseInit = { status: responseInit };
  } else if (typeof responseInit.status === "undefined") {
    responseInit.status = 302;
  }
  const headers = new Headers(responseInit.headers);
  headers.set("Location", url);
  return new Response(null, {
    ...responseInit,
    headers
  });
};
var redirectStatusCodes = /* @__PURE__ */ new Set([301, 302, 303, 307, 308]);
function isRedirectResponse(response) {
  return redirectStatusCodes.has(response.status);
}

// src/node/app/http/http.ts
function createHTTPRequestHandler(urlPattern, loader, options) {
  let _pattern;
  const _getBase = (options == null ? void 0 : options.getBase) ?? ((req) => `https://${req.headers.host}`);
  const _getClientAddress = (options == null ? void 0 : options.getClientAddress) ?? ((req) => req.socket);
  const _handleUnknownError = (options == null ? void 0 : options.handleUnknownError) ?? ((_, res) => {
    res.statusCode = 500;
    res.end("Internal server error");
  });
  return async (req, res) => {
    var _a;
    await ((_a = options == null ? void 0 : options.installPolyfills) == null ? void 0 : _a.call(options));
    let request;
    try {
      request = await getRequest(_getBase(req), req);
    } catch (error) {
      res.statusCode = error.status || 400;
      res.end(error.reason || "Invalid request body");
      return;
    }
    try {
      const response = await handleRequest(
        request,
        _pattern ??= urlPattern(),
        () => _getClientAddress(req),
        loader,
        options == null ? void 0 : options.methods
      );
      setResponse(res, response);
    } catch (error) {
      if (isHTTPError(error)) {
        setResponse(res, handleHTTPError(error));
      } else {
        _handleUnknownError(req, res, error);
      }
    }
  };
}
function handleHTTPError(error) {
  if (isHTTPJSONError(error)) {
    return json(error.data, error.statusCode);
  } else if (isHTTPError(error)) {
    return new Response(error.message, { status: error.statusCode });
  } else {
    return new Response("Internal server error", { status: 500 });
  }
}
var HTTP_METHODS = /* @__PURE__ */ new Set([
  "GET",
  "POST",
  "PUT",
  "PATCH",
  "DELETE"
]);
export {
  HTTPError,
  HTTPJSONError,
  HTTP_METHODS,
  createHTTPRequestHandler,
  createRequestEvent,
  handleHTTPError,
  handleRequest,
  httpError,
  httpJSONError,
  installURLPattern,
  invariant,
  isHTTPError,
  isHTTPJSONError,
  isRedirectResponse,
  json,
  redirect,
  validate
};
